'use strict'

import React from 'react'
import PropTypes from 'prop-types'
import c from 'classnames'
import { EOL } from 'os'

import { TheBodyStyle } from 'the-body'
import { TheBarStyle } from 'the-bar'
import { TheButtonStyle } from 'the-button'
import { TheCardStyle } from 'the-card'
import { TheContainerStyle } from 'the-container'
import { TheDialogStyle } from 'the-dialog'
import { TheDoneStyle } from 'the-done'
import { TheConditionStyle } from 'the-condition'
import { TheDrawerStyle } from 'the-drawer'
import { TheFlickStyle } from 'the-flick'
import { TheLeadStyle } from 'the-lead'
import { TheLineStyle } from 'the-line'
import { TheFooterStyle } from 'the-footer'
import { TheFormStyle } from 'the-form'
import { TheHeaderStyle } from 'the-header'
import { TheIconStyle } from 'the-icon'
import { TheInfoStyle } from 'the-info'
import { TheImageStyle } from 'the-image'
import { TheInputStyle } from 'the-input'
import { TheJumbotronStyle } from 'the-jumbotron'
import { TheLinkStyle } from 'the-link'
import { TheListStyle } from 'the-list'
import { TheMainStyle } from 'the-main'
import { TheMarkdownStyle } from 'the-markdown'
import { TheMenuStyle } from 'the-menu'
import { TheRootStyle } from 'the-root'
import { TheStyle } from 'the-style'
import { TheSectionStyle } from 'the-section'
import { TheSpinStyle } from 'the-spin'
import { TheTabStyle } from 'the-tab'
import { TheTableStyle } from 'the-table'
import { TheTagStyle } from 'the-tag'
import { TheToastStyle } from 'the-toast'
import { TheViewStyle } from 'the-view'
import { TheVideoStyle } from 'the-video'
import { ThePagerStyle } from 'the-pager'

/** Theme Style of the-components */
const TheThemeStyle = ({ id, className, prefix, options }) => (
  <TheStyle {...{ id, prefix }}
            className={c('the-theme-style', className)}
            styles={TheThemeStyle.data(options)}
  />
)

TheThemeStyle.displayName = 'TheThemeStyle'
TheThemeStyle.propTypes = {
  /** Style options */
  options: PropTypes.object,

  /** Style selector prefix */
  prefix: PropTypes.string
}

TheThemeStyle.defaultProps = {
  options: {},
  prefix: null
}

TheThemeStyle.data = (options) => {
  return Object.assign({},
    ...[
      TheBodyStyle,
      TheBarStyle,
      TheButtonStyle,
      TheCardStyle,
      TheContainerStyle,
      TheDialogStyle,
      TheDoneStyle,
      TheConditionStyle,
      TheDrawerStyle,
      TheFlickStyle,
      TheLeadStyle,
      TheLineStyle,
      TheFooterStyle,
      TheFormStyle,
      TheHeaderStyle,
      TheIconStyle,
      TheInfoStyle,
      TheImageStyle,
      TheInputStyle,
      TheJumbotronStyle,
      TheLinkStyle,
      TheListStyle,
      TheMainStyle,
      TheMarkdownStyle,
      TheMenuStyle,
      TheRootStyle,
      TheStyle,
      TheSectionStyle,
      TheSpinStyle,
      TheTabStyle,
      TheTableStyle,
      TheTagStyle,
      TheToastStyle,
      TheViewStyle,
      TheVideoStyle,
      ThePagerStyle
    ].filter((Component) => Component.data).map((Component) => Component.data(options))
  )
}

TheThemeStyle.cssString = (options) => {
  const styles = TheThemeStyle.data(options)
  return Object
    .keys(styles)
    .map((selector) => TheStyle.styleString(selector, styles[ selector ]))
    .filter(Boolean)
    .join(EOL)
}

export default TheThemeStyle
